# Timetable Widget, ZPT widget that renders complex timetables
#
# Copyright (C) 2000-2002 Juan David Ibez Palomar <jdavid@nuxeo.com>
#               2000-2001 Universitat Jaume I (http://www.uji.es)
#               2002      Nuxeo (http://www.nuxeo.com)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""Timetable Widget"""
__version__ = "$Revision: 1.1.1.1 $"



######################################################################
# Security stuff
######################################################################

from AccessControl import ModuleSecurityInfo
__roles__ = None
security = ModuleSecurityInfo()


######################################################################
# Multilingual support
######################################################################

from Globals import package_home
from Products.Localizer.LocalObjects import gettext

class Multilingual:
    locale = package_home(globals()) + '/locale/'
    gettext = gettext

multilingual = Multilingual()


######################################################################
# The meet is here
######################################################################

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from mx import DateTime

# The template with the macro
calendar = PageTemplateFile('calendar', globals())


# Auxiliar logic
def get_date(date):
    """
    Returns today if date is None, otherwise returns the date.

    TODO, some parsing should be implemented.
    """

    if date is None:
        return DateTime.today()

##    date = DateTime.DateTime(date)

    return date


def matrix(date=None, show=1):
    """
    Returns a matrix of days.
    """

    # Get the date
    date = get_date(date)

    # If show is month, show the whole month
    if show == 'month':
        date = DateTime.DateTime(date.year, date.month, 1)
        show = 6

    # Start with the first day of the week
    dow = date.day_of_week
    date = date - dow

    # Build the matrix
    a = []
    for i in range(int(show)):
        b = []
        for j in range(7):
            b.append(date)
            date = date + 1
        a.append(b)

    return a


def weekday(weekday):
    """
    For a weekday number (0 is Monday, 1 is Tuesday, .., 6 is Sunday)
    returns its name, in the correct language.
    """

    name = DateTime.Weekday[weekday]
    return multilingual.gettext(name)

def day_class(date, calendar_date, show):
    """
    Returns the CSS class, which is a space separated list of words. As input
    this method receives:

     - 'date', the date of the cell being rendered;

     - 'calendar_date', the date that the calendar is showing;

     - 'show', the number of weeks that are show.

    The return value always contains the word 'day'. Besides it the
    following words could be included too:

      - 'today', if 'date' is today;

      - 'current', if 'date' and 'calendar_date' are the same;

      - 'in', if 'show' is 'month' and 'date' belongs to the same month
        of 'calendar_date';

      - 'out', if 'show' is 'month' and 'date' doesn't belong to the same
        month of 'calendar_date';

      - 'even', if 'show' is not 'month' and the month number of the date
        is even;

      - 'odd', if 'show' is not 'month' and if the month number of the date
        is even;

    Note that the following combinations are impossible: 'current' and 'out',
    'even' and 'odd', 'even' and 'out', 'odd' and 'out'.
    """

    value = ['day']

    # Is today?
    today = DateTime.today()
    today = date.absdate == today.absdate
    if today:
        value.append('today')

    # Is the current date?
    calendar_date = get_date(calendar_date)
    current = date.absdate == calendar_date.absdate
    if current:
        value.append('current')

    if show == 'month':
        # Out or in the month?
        if date.month != calendar_date.month:
            value.append('out')
        else:
            value.append('in')
    else:
        # Is the month number of date even or odd?
        if date.month % 2:
            value.append('odd')
        else:
            value.append('even')

    # Return..
    return ' '.join(value)


######################################################################
# Security stuff
######################################################################

calendar.__roles__ = None
security.declarePublic('matrix', 'weekday', 'day_class')
security.apply(globals())


DateTime.__roles__ = None
