# Timetable Widget, ZPT widget that renders complex timetables
#
# Copyright (C) 2000-2002 Juan David Ibez Palomar <jdavid@nuxeo.com>
#               2000-2001 Universitat Jaume I (http://www.uji.es)
#               2002      Nuxeo (http://www.nuxeo.com)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""Timetable Widget"""
__version__ = "$Revision: 1.1.1.1 $"




######################################################################
# Security stuff
######################################################################

from AccessControl import ModuleSecurityInfo
__roles__ = None
security = ModuleSecurityInfo()



######################################################################
# The meet is here
######################################################################

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# The template with the macro
timetable = PageTemplateFile('timetable', globals())


from OldCode import Time, TimeGrid, TimeGridsCollection, Row, Cell

TimeGridsCollection.__allow_access_to_unprotected_subobjects__ = 1
Row.__allow_access_to_unprotected_subobjects__ = 1
Cell.__allow_access_to_unprotected_subobjects__ = 1


def get_data(data, grid):
    """ """

    # Provide default values for the parameters
    if data is None:
        data = []

    if grid is None:
        grid = []
    else:
        grid = [ Time(x) for x in grid ]

    # Build..
    headers = []
    events_with_time = []
    events_without_time = []
    for column in data:
        # Initialize
        time_grid = TimeGrid()
        full_day = []

        # Get the headers
        try:
            header = column.header
        except AttributeError:
            try:
                header = column['header']
            except:
                header = None

        headers.append(header)


        # Get the events
        try:
            events = tuple(column)
        except TypeError:
            try:
                events = column.events
            except AttributeError:
                events = column['events']

        # Build the time grid
        for event in events:
            # Get the start and end times, try first with an attirbute
            # if it fails try as a mapping (note, perhaps the mapping
            # should be removed).
            if hasattr(event, 'start'):
                start = event.start
            else:
                start = event['start']

            if hasattr(event, 'end'):
                end = event.end
            else:
                end = event['end']            

            # Start and end could be functions, if they're call them to
            # get the values.
            if callable(start):
                start = start()

            if callable(end):
                end = end()

            # Put the event in the right place
            try:
                start, end = Time(start), Time(end)
            except:
                full_day.append(event)
            else:
                time_grid.set(start, end, event)

                # Fix grid if needed
                if start not in grid:
                    grid.append(start)
                if end not in grid:
                    grid.append(end)

        # Finalize
        events_with_time.append(time_grid)
        events_without_time.append(full_day)


    # Sort the grid
    grid.sort()


    return TimeGridsCollection(events_with_time, headers, grid,
                               events_without_time)





######################################################################
# Security stuff
######################################################################

timetable.__roles__ = None
security.declarePublic('get_data')
security.apply(globals())
